/*
 * Copyright (C) 2000-2006 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

/**
 * @file cm_local.h
 *
 * @brief Implementation of a simple connection manager
 *  
 * \author Mustafa Yuecel <mustafa.yuecel@alumni.ethz.ch>
 * 
 * \date 2006/04/06
 */

#ifndef CM_LOCAL_H
#define CM_LOCAL_H

#include <bt/bt_hci_api.h>
#include <bt/bt_psm.h>
#include <sys/types.h>

#include <cm/con_mgr.h>
#define con_mgr_init cm_local_init

/* Bit Mask used to determine which bits of the Class of Device CoD
 * are 'dont care': Zero-value bits in the mask indicate the 'dont care' bits.
 * Used in filtering and reducing inquiry results (set_event_filter).
 * Note: long format required
 */
#define CM_COD_MASK     0xFFFFFFl

// BT address prefix of the BTnodes
#define CM_BT_ADDR5     0x00
#define CM_BT_ADDR4     0x04
#define CM_BT_ADDR3     0x3F
#define CM_BT_ADDR2     0x00

// max. devices per inquiry
#define CM_INQ_MAX_DEVS 10

// default link policy (hold mode disabled, sniff mode enabled and role switch allowed)
#define CM_DEFAULT_LINK_POLICY BT_HCI_LINK_POLICY_ALL_DISABLED | BT_HCI_LINK_POLICY_ROLE_SWITCH | BT_HCI_LINK_POLICY_SNIFF_MODE

// EEPROM PARAMS --->
/** priority of this thread (see #NutThreadSetPriority) */
#define CM_DEFAULT_THREAD_PRIORITY                    100  // 1 highest, 254 lowest, 64 default

/** sleep time in ms */
#define CM_DEFAULT_MIN_SLEEP_TIME                     10*1000l
#define CM_DEFAULT_MAX_SLEEP_TIME                     120*1000l
#define CM_DEFAULT_STEP_SLEEP_TIME                    10*1000l

/** inquiry duration (x * 1.28s) */
#define CM_DEFAULT_INQ_DURATION                       2
/** preferred scan mode while inquiring */ 
#define CM_DEFAULT_INQ_SCAN_MODE                      BT_HCI_SCAN_MODE_OFF

/** connect timeout (x * 0.625ms) */
#define CM_DEFAULT_PAGE_TIMEOUT                       3200
/** preferred scan mode while paging */
#define CM_DEFAULT_PAGE_SCAN_MODE                     BT_HCI_SCAN_MODE_OFF

/** duration in ms to reject new connections after be connected */
#define CM_DEFAULT_SCAN_MODE_TIMEOUT                  0
//#define CM_DEFAULT_SCAN_MODE_TIMEOUT                  90*1000l // ms, should be greater than CM_DEFAULT_PAGE_LIST_TIME_OUT or 0

/* expiry time for the page list in s */
#define CM_DEFAULT_PAGE_LIST_EXP_TIME                 120*2
/* expiry time for the page blacklist in s */
#define CM_DEFAULT_PAGE_BLACKLIST_EXP_TIME            120*3

/* maximum page retries until blacklisting */
#define CM_DEFAULT_PAGE_MAX_PAGE_ATTEMPTS             2

/* limit of connections */
#define CM_DEFAULT_MY_ROLE_MASTER_MAX_OWN_DEVICES     2  // my CoD == remote CoD, only supports 2/3 sniffing links
#define CM_DEFAULT_MY_ROLE_MASTER_MAX_FOREIGN_DEVICES 6  // my CoD != remote CoD
#define CM_DEFAULT_MY_ROLE_SLAVE_MAX_OWN_DEVICES      1  // my CoD == remote CoD, only supports 1 sniffing links
#define CM_DEFAULT_MY_ROLE_SLAVE_MAX_FOREIGN_DEVICES  2  // my CoD != remote CoD 
// EEPROM PARAMS <---

/**
 * Initialization function for the connection manger.
 */
void cm_local_init(struct btstack * bt_stack, bt_psm_t * psmux, u_short cm_psm, HCI_CON_TABLE_CB_REGISTRATION, u_long cod);

#endif
